<?php

class EveryBoard_Settings {

    private $_settings;
    protected static $templates = [];
    private static $available_templates = null;
    private static $article_templates = null;
    private static $available_image_templates = [];
    
    public function __construct() {
        add_action( 'wp_ajax_ebs_show_form', array( &$this, 'form' ) ); //Ajax function for text search
        add_action( 'wp_ajax_ebs_get_available_classes', array( &$this, 'ebs_get_available_classes' ) ); // Test Ajax function for for fetching available classes
        
        static::setup_article_templates();
    }
    
    /**
     * Fetch and match all templates in the current theme
     *
     * @since 0.1
     * @return array
     */
    protected static function setup_article_templates() {
        
        if ( is_null( static::$article_templates ) ) {
    
            self::add_default_template_paths();
        
            $available_templates = array();
            foreach( static::$templates as $path ) {
                if (file_exists( $path ) && $handle = opendir( $path ) ) {
                    while (false !== ($entry = readdir($handle))) {
                        if(preg_match("/\.php$/i", $entry)) {
                            $url = $path."/".$entry;
                            $raw = file_get_contents($url);
                        
                            if(preg_match("/Article name:([a-zåäöÅÄÖ0-9\s\-\_]{3,})/i", $raw, $result)) {
                                $available_templates[trim($result[1])] = basename($url);
                            }
                        
                            if(preg_match("/OC Template name:([a-zåäöÅÄÖ0-9\s\-\_]{3,})/i", $raw, $result)) {
                                $available_templates[trim($result[1])] = basename($url);
                            }
                        }
                    }
                }
            }
        
            ksort( $available_templates );
            static::$article_templates = $available_templates;
        }
        
        return static::$article_templates;
    }

    /**
     * Ajax function to get the available css-classes added by the user
     */
    function ebs_get_available_classes() {
        $cssObj = new EveryBoard_Global_Settings(false);
        $available_css = $cssObj->eb_global_settings_get_css_classes();

        echo json_encode(isset( $available_css ) ? $available_css : array());
        die();
    }

    public function form() {
        $this->_settings = $_POST["settings"];

        if( isset( $this->_settings['css_class'] ) && !is_array( $this->_settings['css_class'] ) ){
            $this->_settings['css_class'] = array( trim( $this->_settings['css_class'] ) );
        }

        foreach($this->_settings as $key => $value) {
            if( !is_array( $this->_settings[$key] ) ){
                $this->_settings[$key] = stripslashes(urldecode($value));
            }else{
                foreach ($this->_settings[$key] as $sub_key => $sub_value) {
                    $this->_settings[$key][$sub_key] = stripslashes(urldecode($sub_value));
                }
            }
        }

        $cssObj = new EveryBoard_Global_Settings(false);
        /*$css = array("None" => "");
        $this->css = array_merge($css, $cssObj->eb_global_settings_get_css_classes());*/
        $available_css = $cssObj->eb_global_settings_get_css_classes();

        $this->css = isset( $available_css ) ? $available_css : array() ;

        echo '<form method="POST" id="board_settings">';

        switch($_POST["type"]) {
            case "row":
                $this->row_form();
                break;
            case "oc_col":
                $this->oc_col_form();
                break;
            case "col":
                $this->col_form();
                break;
            case "oc_article":
                $this->article_form();
                break;
            case "widget":
                $this->widget_form();
                break;
            case "embed_widget":
                $this->embed_widget_form();
                break;
            case "linked_widget":
                $this->linked_widget_form();
                break;
            default:
                echo "No form avail for ".$_POST["type"]."...";
                break;
        }

        echo '<br /><br /><input type="submit" value="Save & close" class="button button-primary">';
        echo '</form>';
        die;
    }

    public function row_form() {
        $this->_input("name", "Name");
        echo '<br />';
        $this->_input("default_message", "Default message");
        echo '<br />';
        /*$this->_select("css_class", "CSS Class", $this->css);
        echo '<br />';*/
        $this->_css_checkbox("css_class", __('CSS Class', 'everyboard'), $this->css);
        echo '<br />';
        $this->_checkbox("ismobile", "Devices", array("Mobile" => "ismobile", "Pad" => "istablet", "Desktop" => "isdesktop"));
        echo '<br /><br />';
        echo '<input type="button" class="save_template" value="' . __('Save as template', 'everyboard') . '">';

    }

    public function oc_col_form() {
        $this->_input("name", __('Name', 'everyboard'));
        echo '<br />';
        $this->_css_checkbox("css_class", __('CSS Class', 'everyboard'), $this->css);
        echo '<br />';
        $this->_textarea("oc_query", __('Query', 'everyboard'));
        echo '<br />';
        $this->_checkbox("ismobile", "Devices", array("Mobile" => "ismobile", "Pad" => "istablet", "Desktop" => "isdesktop"));
    }

    public function col_form() {
        $this->_input("name", __('Name', 'everyboard'));
        echo '<br />';
        $this->_css_checkbox("css_class", __('CSS Class', 'everyboard'), $this->css);
        echo '<br />';
        $this->_checkbox("ismobile", "Devices", array("Mobile" => "ismobile", "Pad" => "istablet", "Desktop" => "isdesktop", "Big Screen" => "isbigscreen"));
        echo '<br />';
        $this->_checkbox("hidewrapper", "Wrapper", array("Deactivate wrapper" => "hidewrapper"));
        echo '<br />';
        $available_templates = EveryBoard_Settings::get_templates();

        $this->_select("layout", __('Base article layout', 'everyboard'), $available_templates);
    }

    public function article_form() {
        $this->_css_checkbox("css_class", "CSS Class", $this->css);
        echo '<br />';
        $this->_checkbox("ismobile", "Devices", array("Mobile" => "ismobile", "Pad" => "istablet", "Desktop" => "isdesktop"));
        echo '<br />';
        $available_templates = EveryBoard_Settings::get_templates();

        $this->_select("layout", "Article layout", $available_templates);

    }

    public function widget_form() {
        $this->_css_checkbox("css_class", __('CSS Class', 'everyboard'), $this->css);
        echo '<br />';
        $this->_checkbox("ismobile", "Devices", array("Mobile" => "ismobile", "Pad" => "istablet", "Desktop" => "isdesktop"));
    }

    public function embed_widget_form() {
        $this->_css_checkbox("css_class", __('CSS Class', 'everyboard'), $this->css);
        echo '<br />';
        $this->_checkbox("ismobile", "Devices", array("Mobile" => "ismobile", "Pad" => "istablet", "Desktop" => "isdesktop"));
        echo '<br />';
        $this->_textarea("content", __('Content', 'everyboard'));
    }

    public function linked_widget_form() {
        $this->_css_checkbox("css_class", __('CSS Class', 'everyboard'), $this->css);
        echo '<br />';
        $this->_checkbox("ismobile", "Devices", array("Mobile" => "ismobile", "Pad" => "istablet", "Desktop" => "isdesktop"));
        echo '<br />';

        $args     = array( 'post_type' => 'everyboard' );
        $loop     = new WP_Query( $args );

        $boards = array("None" => "0");
        while ( $loop->have_posts() ) : $loop->the_post();
            $boards[get_the_title( '', '', false )] = get_the_ID();
            #print '<option value="' . get_the_ID() . '" ' . ( $active_board_id === get_the_ID() ? $selected : '' ) . '>' . the_title( '', '', false ) . '</option>';
        endwhile;

        $this->_select("board", "Linked board", $boards);
    }

    /* HELPER FUNCTIONS */

    /**
     * Function to render input field
     * @param  [type] $name  [description]
     * @param  [type] $title [description]
     * @return [type]        [description]
     */
    private function _input($name, $title) {
        echo '<label for="'.$name.'"><strong>'.$title.'</strong></label><br />';
        echo '<input type="text" value="' . (isset($this->_settings[$name]) ? $this->_settings[$name] : '') . '" name="'.$name.'" id="'.$name.'" placeholder="'.$name.'" />';
    }

    /**
     * Function to render Select boxes
     * @param  [type] $name   [description]
     * @param  [type] $title  [description]
     * @param  Array  $values [description]
     * @return [type]         [description]
     */
    private function _select($name, $title, Array $values) {
        echo '<label for="'.$name.'"><strong>'.$title.'</strong></label><br />';
        echo '<select name="'.$name.'" id="'.$name.'">';
        foreach($values as $key => $value) {
            echo '<option value="'.$value.'" ' . (isset($this->_settings[$name]) && $this->_settings[$name] == $value ? 'selected="selected"' : '') . '>'.$key.'</option>';
        }
        echo '</select>';
    }

    /**
     * Function to render Textarea
     * @param  [type] $name  [description]
     * @param  [type] $title [description]
     * @return [type]        [description]
     */
    private function _textarea($name, $title) {
        echo '<label for="'.$name.'"><strong>'.$title.'</strong></label><br />';
        echo '<textarea name="'.$name.'" id="'.$name.'">'.(isset($this->_settings[$name]) ? $this->_settings[$name] : '') . '</textarea>';
    }

    /**
     * Function to render Checkboxes
     * @param  [type] $name   [description]
     * @param  [type] $title  [description]
     * @param  Array  $values [description]
     * @return [type]         [description]
     */
    private function _checkbox($name, $title, Array $values) {
        echo '<label for="'.$name.'"><strong>'.$title.'</strong></label><br />';
        foreach($values as $key => $value) {
            echo '<input type="checkbox" value="true" name="'.$value.'" ' . ( isset($this->_settings[$value]) || isset($this->_settings[$value]) && $this->_settings[$value] == "true" ? 'checked="checked"' : '') . '> '.$key.' ';
        }
    }

    private function _css_checkbox($name, $title, Array $values) {
        $selected = '';
        echo '<label for="'.$name.'"><strong>'.$title.'</strong></label><br />';
        foreach($values as $key => $value) {
            if( isset( $this->_settings[$name] ) && is_array( $this->_settings[$name] ) && in_array( $value, $this->_settings[$name] ) ){
                $selected = 'checked="checked"';
            }
            echo '<input type="checkbox" name="'.$name.'[]" value="'.$value.'" '.$selected.'  /> '. $key . ' ';
            $selected = '';
        }
    }

    static function get_image_templates( $available_templates = null ) {

        // If available templates has already been set, just return them.
        if( isset( self::$available_image_templates ) && count( self::$available_image_templates ) > 0 ) {
            return self::$available_image_templates;
        }

        self::add_default_template_paths();

        if( !isset( $available_templates ) ){
            $available_templates = array("None" => "None");
        }

        $available_templates_temp = array();
        foreach( self::$templates as $path ) {
            if (file_exists( $path ) && $handle = opendir( $path ) ) {
                while (false !== ($entry = readdir($handle))) {
                    if(preg_match("/\.php$/i", $entry)) {
                        $url = $path."/".$entry;
                        $raw = file_get_contents($url);

                        if(preg_match("/Image name:([a-zåäöÅÄÖ0-9\s\-\_]{3,})/i", $raw, $result)) {
                            $available_templates_temp[trim($result[1])] = basename($url);
                        }
                    }
                }

            }
        }

        ksort( $available_templates_temp );
        $available_templates = array_merge( $available_templates, $available_templates_temp );
        self::$available_image_templates = $available_templates;

        return $available_templates;
    }

    /**
     * Function to get Available templates and show them in Boards
     * @return [Array] [Available templates]
     */
    static function get_templates( $available_templates = null ) {

        if ( is_null(static::$available_templates ) || $available_templates ) {
            $available_templates_temp = static::get_all_templates();
    
            if( !isset( $available_templates ) ) {
                $available_templates = array("None" => "None");
            }
    
            $template_settings = get_option('template_settings', []);
    
            if ( empty($template_settings) ) {
                $template_settings = $available_templates_temp;
            }
    
            $diff = array_diff($available_templates_temp, $template_settings);
    
            foreach (array_keys($diff) as $key) {
                unset($available_templates_temp[$key]);
            }
    
            ksort( $available_templates_temp );
            $available_templates = array_merge( $available_templates, $available_templates_temp );
    
            static::$available_templates = $available_templates;
        }
        
        return static::$available_templates;
    }
    
    /**
     * Function to get all available templates and show them in Board settings
     * @return [Array] [Available templates]
     */
    static function get_all_templates() {
        static::setup_article_templates();
        
        return static::$article_templates;
    }

    /**
     * Function to return the part of an article template
     * @param  [String] $template_name [File name for article template]
     * @return [String]           [Real path to article template]
     */
    static function get_template_path( $template_name ) {
        if( file_exists($template_name)) {
            return $template_name;
        }
        self::add_default_template_paths();

        foreach( self::$templates as $path ) {
            $p = $path."/".$template_name;
            if( file_exists( $p ) ) {
                return $p;
            }
        }

        return '';
    }

    /**
     * Function to migrate old article template paths to new
     * @return [Void]           [description]
     */
    static function migrate_template_path( ) {
        return null;
    }

    /**
     * Function to decode and strip slashes in settings array or object
     * @param  [type] $settings [description]
     * @return [type]           [description]
     */
    static function decode_settings_array( $settings ){

        if( is_array( $settings ) ) {
            foreach( $settings as $key => $value ) {
                if( !is_array( $settings[$key] ) ) {
                    $settings[$key] = stripslashes(urldecode($value));
                }
                else {
                    foreach ($settings[$key] as $sub_key => $sub_value) {
                        $settings[$key][$sub_key] = stripslashes(urldecode($sub_value));
                    }
                }
            }
        }
        elseif( is_object( $settings ) ) {
            foreach($settings as $key => $value) {
                if( !is_array( $settings->$key ) && !is_object( $settings->$key )  ) {
                    $settings->$key = stripslashes(urldecode($value));
                }
                else {
                    foreach ($settings->$key as $sub_key => $sub_value) {
                        $settings->$key->$sub_key = stripslashes(urldecode($sub_value));
                    }
                }
            }
        }

        return $settings;
    }
    
    /**
     * Add a template path to templates property
     * @param $path
     */
    public static function add_template_path($path) {
        if(!in_array($path, self::$templates)) {
            self::$templates[] = $path;
        }
    }
    
    /**
     * Add default paths to templates
     */
    protected static function add_default_template_paths() {
        $paths = [
            get_stylesheet_directory(),
            get_template_directory(),
            get_stylesheet_directory() . '/templates/',
            get_template_directory() . '/templates/'
        ];
        
        foreach ($paths as $p) {
            static::add_template_path($p);
        }
    }
}